@@ -11,6 +11,7 @@ ldns_dnssec_rrs_new()
 {
 	ldns_dnssec_rrs *new_rrs;
 	new_rrs = LDNS_MALLOC(ldns_dnssec_rrs);
+        if(!new_rrs) return NULL;
 	new_rrs->rr = NULL;
 	new_rrs->next = NULL;
 	return new_rrs;
@@ -58,7 +59,7 @@ ldns_dnssec_rrs_add_rr(ldns_dnssec_rrs *rrs, ldns_rr *rr)
 	/* should we error on equal? */
 	if (cmp <= 0) {
 		if (rrs->next) {
-			ldns_dnssec_rrs_add_rr(rrs->next, rr);
+			return ldns_dnssec_rrs_add_rr(rrs->next, rr);
 		} else {
 			new_rrs = ldns_dnssec_rrs_new();
 			new_rrs->rr = rr;
@@ -96,6 +97,7 @@ ldns_dnssec_rrsets_new()
 {
 	ldns_dnssec_rrsets *new_rrsets;
 	new_rrsets = LDNS_MALLOC(ldns_dnssec_rrsets);
+        if(!new_rrsets) return NULL;
 	new_rrsets->rrs = NULL;
 	new_rrsets->type = 0;
 	new_rrsets->signatures = NULL;
@@ -295,10 +297,12 @@ ldns_dnssec_name_new()
 {
 	ldns_dnssec_name *new_name;
 
-	new_name = LDNS_MALLOC(ldns_dnssec_name);
+	new_name = LDNS_CALLOC(ldns_dnssec_name, 1);
 	if (!new_name) {
 		return NULL;
 	}
+	/*
+	 * not needed anymore because CALLOC initalizes everything to zero.
 
 	new_name->name = NULL;
 	new_name->rrsets = NULL;
@@ -309,6 +313,7 @@ ldns_dnssec_name_new()
 	new_name->is_glue = false;
 	new_name->hashed_name = NULL;
 
+	 */
 	return new_name;
 }
 
@@ -318,7 +323,10 @@ ldns_dnssec_name_new_frm_rr(ldns_rr *rr)
 	ldns_dnssec_name *new_name = ldns_dnssec_name_new();
 
 	new_name->name = ldns_rr_owner(rr);
-	ldns_dnssec_name_add_rr(new_name, rr);
+	if(ldns_dnssec_name_add_rr(new_name, rr) != LDNS_STATUS_OK) {
+		ldns_dnssec_name_free(new_name);
+		return NULL;
+	}
 
 	return new_name;
 }
@@ -370,6 +378,15 @@ ldns_dnssec_name_name(ldns_dnssec_name *name)
 	return NULL;
 }
 
+bool
+ldns_dnssec_name_is_glue(ldns_dnssec_name *name)
+{
+	if (name) {
+		return name->is_glue;
+	}
+	return false;
+}
+
 void
 ldns_dnssec_name_set_name(ldns_dnssec_name *rrset,
 					 ldns_rdf *dname)
@@ -421,7 +438,7 @@ ldns_dnssec_name_add_rr(ldns_dnssec_name *name,
 	ldns_status result = LDNS_STATUS_OK;
 	ldns_rdf *name_name;
 	bool hashed_name = false;
-	ldns_rr_type rr_type = ldns_rr_get_type(rr);
+	ldns_rr_type rr_type;
 	ldns_rr_type typecovered = 0;
 
 	/* special handling for NSEC3 and NSECX covering RRSIGS */
@@ -456,7 +473,7 @@ ldns_dnssec_name_add_rr(ldns_dnssec_name *name,
 	} else if (typecovered == LDNS_RR_TYPE_NSEC ||
 			 typecovered == LDNS_RR_TYPE_NSEC3) {
 		if (name->nsec_signatures) {
-			ldns_dnssec_rrs_add_rr(name->nsec_signatures, rr);
+			result = ldns_dnssec_rrs_add_rr(name->nsec_signatures, rr);
 		} else {
 			name->nsec_signatures = ldns_dnssec_rrs_new();
 			name->nsec_signatures->rr = rr;
@@ -514,15 +531,6 @@ ldns_dnssec_zone_find_rrset(ldns_dnssec_zone *zone,
 	}
 }
 
-static inline void
-print_indent(FILE *out, int c)
-{
-	int i;
-	for (i=0; i<c; i++) {
-		fprintf(out, "    ");
-	}
-}
-
 void
 ldns_dnssec_name_print_soa(FILE *out, ldns_dnssec_name *name, bool show_soa)
 {
@@ -555,6 +563,7 @@ ldns_dnssec_zone *
 ldns_dnssec_zone_new()
 {
 	ldns_dnssec_zone *zone = LDNS_MALLOC(ldns_dnssec_zone);
+        if(!zone) return NULL;
 	zone->soa = NULL;
 	zone->names = NULL;
 
@@ -653,6 +662,7 @@ ldns_dnssec_zone_add_rr(ldns_dnssec_zone *zone, ldns_rr *rr)
 
 	if (!zone->names) {
 		zone->names = ldns_rbtree_create(ldns_dname_compare_v);
+                if(!zone->names) return LDNS_STATUS_MEM_ERR;
 	}
 
 	/* we need the original of the hashed name if this is
@@ -674,13 +684,18 @@ ldns_dnssec_zone_add_rr(ldns_dnssec_zone *zone, ldns_rr *rr)
 	if (!cur_node) {
 		/* add */
 		cur_name = ldns_dnssec_name_new_frm_rr(rr);
+                if(!cur_name) return LDNS_STATUS_MEM_ERR;
 		cur_node = LDNS_MALLOC(ldns_rbnode_t);
+                if(!cur_node) {
+                        ldns_dnssec_name_free(cur_name);
+                        return LDNS_STATUS_MEM_ERR;
+                }
 		cur_node->key = ldns_rr_owner(rr);
 		cur_node->data = cur_name;
-		ldns_rbtree_insert(zone->names, cur_node);
+		(void)ldns_rbtree_insert(zone->names, cur_node);
 	} else {
 		cur_name = (ldns_dnssec_name *) cur_node->data;
-		ldns_dnssec_name_add_rr(cur_name, rr);
+		result = ldns_dnssec_name_add_rr(cur_name, rr);
 	}
 
 	if (result != LDNS_STATUS_OK) {
@@ -784,13 +799,13 @@ ldns_dnssec_zone_add_empty_nonterminals(ldns_dnssec_zone *zone)
 		 * label in the current name (counting from the end)
 		 */
 		for (i = 1; i < next_label_count - soa_label_count; i++) {
-			lpos = cur_label_count - next_label_count + i;
+			lpos = (int)cur_label_count - (int)next_label_count + (int)i;
 			if (lpos >= 0) {
-				l1 = ldns_dname_label(cur_name, lpos);
+				l1 = ldns_dname_clone_from(cur_name, (uint8_t)lpos);
 			} else {
 				l1 = NULL;
 			}
-			l2 = ldns_dname_label(next_name, i);
+			l2 = ldns_dname_clone_from(next_name, i);
 
 			if (!l1 || ldns_dname_compare(l1, l2) != 0) {
 				/* We have an empty nonterminal, add it to the
@@ -802,7 +817,7 @@ ldns_dnssec_zone_add_empty_nonterminals(ldns_dnssec_zone *zone)
 				}
 				new_name->name = ldns_dname_clone_from(next_name,
 				                                       i);
-				if (!new_name) {
+				if (!new_name->name) {
 					ldns_dnssec_name_free(new_name);
 					return LDNS_STATUS_MEM_ERR;
 				}
@@ -814,7 +829,7 @@ ldns_dnssec_zone_add_empty_nonterminals(ldns_dnssec_zone *zone)
 				}
 				new_node->key = new_name->name;
 				new_node->data = new_name;
-				ldns_rbtree_insert(zone->names, new_node);
+				(void)ldns_rbtree_insert(zone->names, new_node);
 			}
 			ldns_rdf_deep_free(l1);
 			ldns_rdf_deep_free(l2);
