@@ -370,7 +370,9 @@ print_match_operation(FILE *output, match_operation *mc)
 					}
 					break;
 				case TYPE_TIMESTAMP:
+#ifndef S_SPLINT_S
 					time.tv_sec = (long int) atol(mc->value);
+#endif
 					time_tt = (time_t)time.tv_sec;
 					tmp = ctime(&time_tt);
 					tmp2 = malloc(strlen(tmp) + 1);
@@ -1850,8 +1852,7 @@ parse_match_expression(char *string)
 									val = malloc(4);
 									snprintf(val, 3, "%u", (unsigned int) lt->id);
 								} else {
-									val = malloc(strlen(str) - i + 1);
-									strcpy(val, &str[i]);
+									val = strdup(&str[i]);
 								}
 								break;
 							case TYPE_RCODE:
@@ -1860,13 +1861,11 @@ parse_match_expression(char *string)
 									val = malloc(4);
 									snprintf(val, 3, "%u", (unsigned int) lt->id);
 								} else {
-									val = malloc(strlen(str) - i + 1);
-									strcpy(val, &str[i]);
+									val = strdup(&str[i]);
 								}
 								break;
 							default:
-								val = malloc(strlen(str) - i + 1);
-								strcpy(val, &str[i]);
+								val = strdup(&str[i]);
 								break;
 						}
 						mo->value = val;
