@@ -21,21 +21,6 @@
 
 int verbosity = 3;
 
-/* returns 1 if the list is empty, or if there are only ns rrs in the
- * list, 0 otherwise */
-static int
-only_ns_in_rrsets(ldns_dnssec_rrsets *rrsets) {
-	ldns_dnssec_rrsets *cur_rrset = rrsets;
-
-	while (cur_rrset) {
-		if (cur_rrset->type != LDNS_RR_TYPE_NS) {
-			return 0;
-		}
-		cur_rrset = cur_rrset->next;
-	}
-	return 1;
-}
-
 static int
 zone_is_nsec3_optout(ldns_rbtree_t *zone_nodes)
 {
@@ -66,11 +51,8 @@ ldns_rr_list_contains_name(const ldns_rr_list *rr_list,
 {
 	size_t i;
 	for (i = 0; i < ldns_rr_list_rr_count(rr_list); i++) {
-		if (ldns_dname_compare(name, 
-						   ldns_rr_owner(ldns_rr_list_rr(rr_list, 
-												   i))
-						   ) == 0
-		    ) {
+		if (ldns_dname_compare(name,
+		    ldns_rr_owner(ldns_rr_list_rr(rr_list, i))) == 0) {
 			return true;
 		}
 	}
@@ -147,17 +129,16 @@ static ldns_status
 verify_dnssec_rrset(ldns_rdf *zone_name,
 					ldns_rdf *name,
                     ldns_dnssec_rrsets *rrset,
-                    ldns_rr_list *keys,
-                    ldns_rr_list *glue_rrs)
+                    ldns_rr_list *keys)
 {
 	ldns_rr_list *rrset_rrs;
 	ldns_dnssec_rrs *cur_rr, *cur_sig;
 	ldns_status status;
 	ldns_rr_list *good_keys;
 	ldns_status result = LDNS_STATUS_OK;
-	
+
 	if (!rrset->rrs) return LDNS_STATUS_OK;
-	
+
 	rrset_rrs = ldns_rr_list_new();
 	cur_rr = rrset->rrs;
 	while(cur_rr && cur_rr->rr) {
@@ -229,7 +210,7 @@ verify_single_rr(ldns_rr *rr,
 	ldns_dnssec_rrs *cur_sig;
 	ldns_status status;
 	ldns_status result = LDNS_STATUS_OK;
-    
+
 	rrset_rrs = ldns_rr_list_new();
 	ldns_rr_list_push_rr(rrset_rrs, rr);
 
@@ -285,7 +266,7 @@ verify_next_hashed_name(ldns_rbtree_t *zone_nodes,
 	int cmp;
 	char *next_owner_str;
 	ldns_rdf *next_owner_dname;
-	
+
 	if (!name->hashed_name) {
 		name->hashed_name = ldns_nsec3_hash_name_frm_nsec3(name->nsec,
 		                                                   name->name);
@@ -338,7 +319,7 @@ verify_next_hashed_name(ldns_rbtree_t *zone_nodes,
 	if (!cur_next_name) {
 		cur_next_name = cur_first_name;
 	}
-	
+
 	next_owner_str = ldns_rdf2str(ldns_nsec3_next_owner(name->nsec));
 	next_owner_dname = ldns_dname_new_frm_str(next_owner_str);
 	cmp = ldns_dname_compare(next_owner_dname,
@@ -360,35 +341,16 @@ verify_next_hashed_name(ldns_rbtree_t *zone_nodes,
 	}
 }
 
-static ldns_rbnode_t *
-next_nonglue_node(ldns_rbnode_t *node, ldns_rr_list *glue_rrs)
-{
-	ldns_rbnode_t *cur_node = ldns_rbtree_next(node);
-	ldns_dnssec_name *cur_name;
-	while (cur_node != LDNS_RBTREE_NULL) {
-		cur_name = (ldns_dnssec_name *) cur_node->data;
-		if (cur_name && cur_name->name) {
-			if (!ldns_rr_list_contains_name(glue_rrs, cur_name->name)) {
-				return cur_node;
-			}
-		}
-		cur_node = ldns_rbtree_next(cur_node);
-	}
-	return LDNS_RBTREE_NULL;
-}
-
 static ldns_status
 verify_nsec(ldns_rbtree_t *zone_nodes,
             ldns_rbnode_t *cur_node,
-            ldns_rr_list *keys,
-            ldns_rr_list *glue_rrs
-)
+            ldns_rr_list *keys)
 {
 	ldns_rbnode_t *next_node;
 	ldns_dnssec_name *name, *next_name;
 	ldns_status status, result;
 	result = LDNS_STATUS_OK;
-	
+
 	name = (ldns_dnssec_name *) cur_node->data;
 	if (name->nsec) {
 		if (name->nsec_signatures) {
@@ -412,10 +374,11 @@ verify_nsec(ldns_rbtree_t *zone_nodes,
 		switch (ldns_rr_get_type(name->nsec)) {
 			case LDNS_RR_TYPE_NSEC:
 				/* simply try next name */
-				next_node = next_nonglue_node(cur_node, glue_rrs);
+				next_node = ldns_rbtree_next(cur_node);
 				if (next_node == LDNS_RBTREE_NULL) {
 					next_node = ldns_rbtree_first(zone_nodes);
 				}
+				next_node = ldns_dnssec_name_node_next_nonglue(next_node);
 				next_name = (ldns_dnssec_name *)next_node->data;
 				if (ldns_dname_compare(next_name->name,
 									   ldns_rr_rdf(name->nsec, 0))
@@ -423,6 +386,13 @@ verify_nsec(ldns_rbtree_t *zone_nodes,
 					printf("Error: the NSEC record for ");
 					ldns_rdf_print(stdout, name->name);
 					printf(" points to the wrong next owner name\n");
+					if (verbosity >= 4) {
+						printf("     : ");
+						ldns_rdf_print(stdout,ldns_rr_rdf(name->nsec, 0));
+						printf(" i.s.o. ");
+						ldns_rdf_print(stdout, next_name->name);
+						printf(".\n");
+					}
 					if (result == LDNS_STATUS_OK) {
 						result = LDNS_STATUS_ERR;
 					}
@@ -442,11 +412,17 @@ verify_nsec(ldns_rbtree_t *zone_nodes,
 			default:
 				break;
 		}
-		
 	} else {
 		/* todo; do this once and cache result? */
 		if (zone_is_nsec3_optout(zone_nodes) &&
-		    only_ns_in_rrsets(name->rrsets)) {
+				(   ldns_dnssec_name_is_glue(name)
+				 || (
+					 ldns_dnssec_rrsets_contains_type(
+						name->rrsets, LDNS_RR_TYPE_NS)
+				     && !ldns_dnssec_rrsets_contains_type(
+						name->rrsets, LDNS_RR_TYPE_DS)
+				    )
+				)) {
 			/* ok, no problem, but we need to remember to check
 			 * whether the chain does not actually point to this
 			 * name later */
@@ -464,34 +440,18 @@ verify_nsec(ldns_rbtree_t *zone_nodes,
 	return result;
 }
 
-static int
-ldns_dnssec_name_has_only_a(ldns_dnssec_name *cur_name)
-{
-	ldns_dnssec_rrsets *cur_rrset;
-	cur_rrset = cur_name->rrsets;
-	while (cur_rrset) {
-		if (cur_rrset->type != LDNS_RR_TYPE_A &&
-			cur_rrset->type != LDNS_RR_TYPE_AAAA) {
-			return 0;
-		} else {
-			cur_rrset = cur_rrset->next;
-		}
-	}
-	return 1;
-}
-
 static ldns_status
 verify_dnssec_name(ldns_rdf *zone_name,
                 ldns_dnssec_zone *zone,
                 ldns_rbtree_t *zone_nodes,
                 ldns_rbnode_t *cur_node,
-			    ldns_rr_list *keys,
-			    ldns_rr_list *glue_rrs)
+			    ldns_rr_list *keys)
 {
 	ldns_status result = LDNS_STATUS_OK;
 	ldns_status status;
 	ldns_dnssec_rrsets *cur_rrset;
 	ldns_dnssec_name *name;
+	int on_delegation_point;
 	/* for NSEC chain checks */
 
 	name = (ldns_dnssec_name *) cur_node->data;
@@ -501,9 +461,7 @@ verify_dnssec_name(ldns_rdf *zone_name,
 		printf("\n");
 	}
 
-	if (ldns_rr_list_contains_name(glue_rrs, name->name) &&
-		ldns_dnssec_name_has_only_a(name)
-	) {
+	if (ldns_dnssec_name_is_glue(name)) {
 		/* glue */
 		cur_rrset = name->rrsets;
 		while (cur_rrset) {
@@ -529,11 +487,26 @@ verify_dnssec_name(ldns_rdf *zone_name,
 		}
 	} else {
 		/* not glue, do real verify */
+
+		on_delegation_point = ldns_dnssec_rrsets_contains_type(
+				name->rrsets, LDNS_RR_TYPE_NS)
+			&& !ldns_dnssec_rrsets_contains_type(
+				name->rrsets, LDNS_RR_TYPE_SOA);
+
 		cur_rrset = name->rrsets;
 		while(cur_rrset) {
-			if (cur_rrset->type != LDNS_RR_TYPE_A ||
-			    !ldns_dnssec_zone_find_rrset(zone, name->name, LDNS_RR_TYPE_NS)) {
-				status = verify_dnssec_rrset(zone_name, name->name, cur_rrset, keys, glue_rrs);
+
+			/* Do not check occluded rrsets
+			 * on the delegation point
+			 */
+			if (       ( on_delegation_point && (
+					cur_rrset->type == LDNS_RR_TYPE_NS
+				     || cur_rrset->type == LDNS_RR_TYPE_DS))
+				|| (!on_delegation_point &&
+					cur_rrset->type != LDNS_RR_TYPE_RRSIG
+				     && cur_rrset->type != LDNS_RR_TYPE_NSEC)) {
+
+				status = verify_dnssec_rrset(zone_name, name->name, cur_rrset, keys);
 				if (status != LDNS_STATUS_OK && result == LDNS_STATUS_OK) {
 					result = status;
 				}
@@ -541,7 +514,7 @@ verify_dnssec_name(ldns_rdf *zone_name,
 			cur_rrset = cur_rrset->next;
 		}
 
-		status = verify_nsec(zone_nodes, cur_node, keys, glue_rrs);
+		status = verify_nsec(zone_nodes, cur_node, keys);
 		if (result == LDNS_STATUS_OK) {
 			result = status;
 		}
@@ -551,9 +524,7 @@ verify_dnssec_name(ldns_rdf *zone_name,
 
 static ldns_status
 verify_dnssec_zone(ldns_dnssec_zone *dnssec_zone,
-			    ldns_rdf *zone_name,
-			    ldns_rr_list *glue_rrs)
-{
+			    ldns_rdf *zone_name) {
 	ldns_rr_list *keys;
 	ldns_rbnode_t *cur_node;
 	ldns_dnssec_rrsets *cur_key_rrset;
@@ -574,6 +545,10 @@ verify_dnssec_zone(ldns_dnssec_zone *dnssec_zone,
 	} else {
 		cur_key = cur_key_rrset->rrs;
 		while (cur_key) {
+			if (verbosity >= 4) {
+				printf("DNSKEY: ");
+				ldns_rr_print(stdout, cur_key->rr);
+			}
 			ldns_rr_list_push_rr(keys, cur_key->rr);
 			cur_key = cur_key->next;
 		}
@@ -591,8 +566,7 @@ verify_dnssec_zone(ldns_dnssec_zone *dnssec_zone,
 			                            dnssec_zone,
 			                            dnssec_zone->names,
 			                            cur_node,
-			                            keys,
-			                            glue_rrs);
+			                            keys);
 			if (status != LDNS_STATUS_OK && result == LDNS_STATUS_OK) {
 				result = status;
 			}
@@ -615,8 +589,7 @@ main(int argc, char **argv)
 	ldns_status s;
 	ldns_dnssec_zone *dnssec_zone;
 	ldns_status result = LDNS_STATUS_ERR;
-	ldns_rr_list *glue_rrs;
-	
+
 	while ((c = getopt(argc, argv, "hvV:")) != -1) {
 		switch(c) {
 		case 'h':
@@ -633,7 +606,7 @@ main(int argc, char **argv)
 			exit(EXIT_SUCCESS);
 			break;
 		case 'v':
-			printf("read zone version %s (ldns version %s)\n", 
+			printf("read zone version %s (ldns version %s)\n",
 				  LDNS_VERSION, ldns_version());
 			exit(EXIT_SUCCESS);
 			break;
@@ -645,7 +618,7 @@ main(int argc, char **argv)
 
 	argc -= optind;
 	argv += optind;
-	
+
 	if (argc == 0) {
 		fp = stdin;
 	} else {
@@ -660,25 +633,29 @@ main(int argc, char **argv)
 			exit(EXIT_FAILURE);
 		}
 	}
-	
+
 	s = ldns_zone_new_frm_fp_l(&z, fp, NULL, 0, LDNS_RR_CLASS_IN, &line_nr);
 
-	if (!ldns_zone_soa(z)) {
-		fprintf(stderr, "; Error: no SOA in the zone\n");
-		exit(1);
-	}
-
-	glue_rrs = ldns_zone_glue_rr_list(z);
 	if (s == LDNS_STATUS_OK) {
+		if (!ldns_zone_soa(z)) {
+			fprintf(stderr, "; Error: no SOA in the zone\n");
+			exit(1);
+		}
+
 		dnssec_zone = create_dnssec_zone(z);
+		result = ldns_dnssec_zone_mark_glue(dnssec_zone);
+		if (result != LDNS_STATUS_OK) {
+			if (verbosity >= 1) {
+				printf("There were errors identifying the glue in the zone\n");
+			}
+		}
 
 		if (verbosity >= 5) {
 			ldns_dnssec_zone_print(stdout, dnssec_zone);
 		}
 
-		result = verify_dnssec_zone(dnssec_zone, 
-							   ldns_rr_owner(ldns_zone_soa(z)),
-							   glue_rrs);
+		result = verify_dnssec_zone(dnssec_zone,
+							   ldns_rr_owner(ldns_zone_soa(z)));
 
 
 		if (result == LDNS_STATUS_OK) {
@@ -694,7 +671,7 @@ main(int argc, char **argv)
 		ldns_zone_free(z);
 		ldns_dnssec_zone_deep_free(dnssec_zone);
 	} else {
-		fprintf(stderr, "%s at %d\n", 
+		fprintf(stderr, "%s at %d\n",
 				ldns_get_errorstr_by_id(s),
 				line_nr);
                 exit(EXIT_FAILURE);
