@@ -117,6 +117,7 @@ ldns_fget_token_l(FILE *f, char *token, const char *delim, size_t limit, int *li
 			if (line_nr) {
 				*line_nr = *line_nr + 1;
 			}
+			*t++ = ' ';
 			prev_c = c;
 			continue;
 		}
@@ -131,13 +132,15 @@ ldns_fget_token_l(FILE *f, char *token, const char *delim, size_t limit, int *li
 			}
 		}
 		if (c != '\0' && c != '\n') {
-			*t++ = c;
 			i++;
 		}
 		if (limit > 0 && i >= limit) {
 			*t = '\0';
 			return -1;
 		}
+		if (c != '\0' && c != '\n') {
+			*t++ = c;
+		}
 		if (c == '\\' && prev_c == '\\')
 			prev_c = 0;
 		else	prev_c = c;
@@ -182,6 +185,8 @@ ldns_fget_keyword_data_l(FILE *f, const char *keyword, const char *k_del, char *
        char *fkeyword;
        ssize_t i;
 
+       if(strlen(keyword) >= LDNS_MAX_KEYWORDLEN)
+               return -1;
        fkeyword = LDNS_XMALLOC(char, LDNS_MAX_KEYWORDLEN);
        if(!fkeyword)
                return -1;
@@ -287,6 +292,7 @@ ldns_bget_token(ldns_buffer *b, char *token, const char *delim, size_t limit)
 
 		if (c == '\n' && p != 0) {
 			/* in parentheses */
+			*t++ = ' ';
 			lc = c;
 			continue;
 		}
@@ -298,12 +304,12 @@ ldns_bget_token(ldns_buffer *b, char *token, const char *delim, size_t limit)
                         }
 		}
 
-		*t++ = c;
 		i++;
 		if (limit > 0 && i >= limit) {
 			*t = '\0';
 			return -1;
 		}
+		*t++ = c;
 
 		if (c == '\\' && lc == '\\') {
 			lc = 0;
@@ -414,6 +420,8 @@ ldns_bget_keyword_data(ldns_buffer *b, const char *keyword, const char *k_del, c
        char *fkeyword;
        ssize_t i;
 
+       if(strlen(keyword) >= LDNS_MAX_KEYWORDLEN)
+               return -1;
        fkeyword = LDNS_XMALLOC(char, LDNS_MAX_KEYWORDLEN);
        if(!fkeyword)
                return -1; /* out of memory */
