@@ -53,6 +53,10 @@ ldns_rr_new_frm_type(ldns_rr_type t)
 	desc = ldns_rr_descript(t);
 
 	rr->_rdata_fields = LDNS_XMALLOC(ldns_rdf *, ldns_rr_descriptor_minimum(desc));
+        if(!rr->_rdata_fields) {
+                LDNS_FREE(rr);
+                return NULL;
+        }
 	for (i = 0; i < ldns_rr_descriptor_minimum(desc); i++) {
 		rr->_rdata_fields[i] = NULL;
 	}
@@ -105,22 +109,24 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 	ldns_rr *new;
 	const ldns_rr_descriptor *desc;
 	ldns_rr_type rr_type;
-	ldns_buffer *rr_buf;
-	ldns_buffer *rd_buf;
+	ldns_buffer *rr_buf = NULL;
+	ldns_buffer *rd_buf = NULL;
 	uint32_t ttl_val;
-	char  *owner;
-	char  *ttl;
+	char  *owner = NULL;
+	char  *ttl = NULL;
 	ldns_rr_class clas_val;
-	char  *clas;
+	char  *clas = NULL;
 	char  *type = NULL;
-	char  *rdata;
-	char  *rd;
-	char  *b64;
+	char  *rdata = NULL;
+	char  *rd = NULL;
+	char  *b64 = NULL;
 	size_t rd_strlen;
 	const char *delimiters;
 	ssize_t c;
 	ldns_rdf *owner_dname;
-    const char* endptr;
+        const char* endptr;
+        int was_unknown_rr_format = 0;
+	ldns_status status = LDNS_STATUS_OK;
 
 	/* used for types with unknown number of rdatas */
 	bool done;
@@ -130,12 +136,7 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 	uint16_t r_cnt;
 	uint16_t r_min;
 	uint16_t r_max;
-
-	uint16_t hex_data_size;
-	char *hex_data_str;
-	uint16_t cur_hex_data_size;
-	uint8_t *hex_data;
-	size_t hex_pos;
+        size_t pre_data_pos;
 
 	new = ldns_rr_new();
 
@@ -148,39 +149,24 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 	rd = LDNS_XMALLOC(char, LDNS_MAX_RDFLEN);
 	b64 = LDNS_XMALLOC(char, LDNS_MAX_RDFLEN);
 	if (!new || !owner || !ttl || !clas || !rdata || !rr_buf || !rd_buf || !rd || !b64 ) {
-		return LDNS_STATUS_MEM_ERR;
+		status = LDNS_STATUS_MEM_ERR;
+		LDNS_FREE(rr_buf);
+		goto ldnserror;
 	}
-	r_cnt = 0;
-	ttl_val = 0;
-	clas_val = 0;
 
 	ldns_buffer_new_frm_data(rr_buf, (char*)str, strlen(str));
 
 	/* split the rr in its parts -1 signals trouble */
 	if (ldns_bget_token(rr_buf, owner, "\t\n ", LDNS_MAX_DOMAINLEN) == -1) {
-		LDNS_FREE(owner);
-		LDNS_FREE(ttl);
-		LDNS_FREE(clas);
-		LDNS_FREE(rdata);
-		LDNS_FREE(rd);
-		LDNS_FREE(rd_buf);
-		LDNS_FREE(b64);
+		status = LDNS_STATUS_SYNTAX_ERR;
 		ldns_buffer_free(rr_buf);
-		ldns_rr_free(new);
-		return LDNS_STATUS_SYNTAX_ERR;
+		goto ldnserror;
 	}
 
 	if (ldns_bget_token(rr_buf, ttl, "\t\n ", LDNS_TTL_DATALEN) == -1) {
-		LDNS_FREE(owner);
-		LDNS_FREE(ttl);
-		LDNS_FREE(clas);
-		LDNS_FREE(rdata);
-		LDNS_FREE(rd);
-		LDNS_FREE(rd_buf);
-		LDNS_FREE(b64);
+		status = LDNS_STATUS_SYNTAX_TTL_ERR;
 		ldns_buffer_free(rr_buf);
-		ldns_rr_free(new);
-		return LDNS_STATUS_SYNTAX_TTL_ERR;
+		goto ldnserror;
 	}
 	ttl_val = (uint32_t) ldns_str2period(ttl, &endptr);
 
@@ -203,19 +189,18 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 		if (clas_val == 0) {
 			clas_val = LDNS_RR_CLASS_IN;
 			type = LDNS_XMALLOC(char, strlen(ttl) + 1);
+			if(!type) {
+				status = LDNS_STATUS_MEM_ERR;
+				ldns_buffer_free(rr_buf);
+				goto ldnserror;
+			}
 			strncpy(type, ttl, strlen(ttl) + 1);
 		}
 	} else {
 		if (ldns_bget_token(rr_buf, clas, "\t\n ", LDNS_SYNTAX_DATALEN) == -1) {
-			LDNS_FREE(owner);
-			LDNS_FREE(ttl);
-			LDNS_FREE(clas);
-			LDNS_FREE(rdata);
-			LDNS_FREE(rd);
-			LDNS_FREE(rd_buf);
+			status = LDNS_STATUS_SYNTAX_CLASS_ERR;
 			ldns_buffer_free(rr_buf);
-			ldns_rr_free(new);
-			return LDNS_STATUS_SYNTAX_CLASS_ERR;
+			goto ldnserror;
 		}
 		clas_val = ldns_get_rr_class_by_name(clas);
 		/* class can be left out too, assume IN, current
@@ -224,6 +209,11 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 		if (clas_val == 0) {
 			clas_val = LDNS_RR_CLASS_IN;
 			type = LDNS_XMALLOC(char, strlen(clas) + 1);
+			if(!type) {
+				status = LDNS_STATUS_MEM_ERR;
+				ldns_buffer_free(rr_buf);
+				goto ldnserror;
+			}
 			strncpy(type, clas, strlen(clas) + 1);
 		}
 	}
@@ -231,32 +221,21 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 
 	if (!type) {
 		type = LDNS_XMALLOC(char, LDNS_SYNTAX_DATALEN);
-		if (ldns_bget_token(rr_buf, type, "\t\n ", LDNS_SYNTAX_DATALEN) == -1) {
-			LDNS_FREE(owner);
-			LDNS_FREE(ttl);
-			LDNS_FREE(clas);
-			LDNS_FREE(rdata);
-			LDNS_FREE(rd);
-			LDNS_FREE(rd_buf);
+		if(!type) {
+			status = LDNS_STATUS_MEM_ERR;
 			ldns_buffer_free(rr_buf);
-			ldns_rr_free(new);
-			return LDNS_STATUS_SYNTAX_TYPE_ERR;
+			goto ldnserror;
+		}
+		if (ldns_bget_token(rr_buf, type, "\t\n ", LDNS_SYNTAX_DATALEN) == -1) {
+			status = LDNS_STATUS_SYNTAX_TYPE_ERR;
+			ldns_buffer_free(rr_buf);
+			goto ldnserror;
 		}
 	}
 
 	if (ldns_bget_token(rr_buf, rdata, "\0", LDNS_MAX_PACKETLEN) == -1) {
 		/* apparently we are done, and it's only a question RR
-		 * so do not free and error here
-		LDNS_FREE(owner);
-		LDNS_FREE(ttl);
-		LDNS_FREE(clas);
-		LDNS_FREE(type);
-		LDNS_FREE(rdata);
-		LDNS_FREE(rd);
-		LDNS_FREE(rd_buf);
-		ldns_buffer_free(rr_buf);
-		ldns_rr_free(new);
-		return NULL;
+		 * so do not set status and go to ldnserror here
 		*/
 	}
 
@@ -276,6 +255,11 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 		if (prev) {
 			ldns_rdf_deep_free(*prev);
 			*prev = ldns_rdf_clone(ldns_rr_owner(new));
+			if (!*prev) {
+				status = LDNS_STATUS_MEM_ERR;
+				ldns_buffer_free(rr_buf);
+				goto ldnserror;
+			}
 		}
 	} else {
 		if (strlen(owner) == 0) {
@@ -288,55 +272,55 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 			} else {
 				ldns_rr_set_owner(new, ldns_dname_new_frm_str("."));
 			}
+			if(!ldns_rr_owner(new)) {
+				status = LDNS_STATUS_MEM_ERR;
+				ldns_buffer_free(rr_buf);
+				goto ldnserror;
+			}
 		} else {
 			owner_dname = ldns_dname_new_frm_str(owner);
 			if (!owner_dname) {
-					LDNS_FREE(owner);
-					LDNS_FREE(ttl);
-					LDNS_FREE(clas);
-					LDNS_FREE(type);
-					LDNS_FREE(rdata);
-					LDNS_FREE(rd);
-					LDNS_FREE(rd_buf);
-					ldns_buffer_free(rr_buf);
-					ldns_rr_free(new);
-					return LDNS_STATUS_SYNTAX_ERR;
+				status = LDNS_STATUS_SYNTAX_ERR;
+				ldns_buffer_free(rr_buf);
+				goto ldnserror;
 			}
 
 			ldns_rr_set_owner(new, owner_dname);
 			if (!ldns_dname_str_absolute(owner) && origin) {
 				if(ldns_dname_cat(ldns_rr_owner(new),
 							origin) != LDNS_STATUS_OK) {
-					LDNS_FREE(owner);
-					LDNS_FREE(ttl);
-					LDNS_FREE(clas);
-					LDNS_FREE(type);
-					LDNS_FREE(rdata);
-					LDNS_FREE(rd);
-					LDNS_FREE(rd_buf);
+					status = LDNS_STATUS_SYNTAX_ERR;
 					ldns_buffer_free(rr_buf);
-					ldns_rr_free(new);
-					return LDNS_STATUS_SYNTAX_ERR;
+					goto ldnserror;
 				}
 			}
 			if (prev) {
 				ldns_rdf_deep_free(*prev);
 				*prev = ldns_rdf_clone(ldns_rr_owner(new));
+				if(!*prev) {
+					status = LDNS_STATUS_MEM_ERR;
+					ldns_buffer_free(rr_buf);
+					goto ldnserror;
+				}
 			}
 		}
 	}
 	LDNS_FREE(owner);
+	owner = NULL;
 
 	ldns_rr_set_question(new, question);
 
 	ldns_rr_set_ttl(new, ttl_val);
 	LDNS_FREE(ttl);
+	ttl = NULL;
 
 	ldns_rr_set_class(new, clas_val);
 	LDNS_FREE(clas);
+	clas = NULL;
 
 	rr_type = ldns_get_rr_type_by_name(type);
 	LDNS_FREE(type);
+	type = NULL;
 
 	desc = ldns_rr_descript((uint16_t)rr_type);
 	ldns_rr_set_type(new, rr_type);
@@ -394,11 +378,12 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 				   rely on _maximum() only */
 				/* skip spaces */
 				while (ldns_buffer_position(rd_buf) < ldns_buffer_limit(rd_buf) &&
-					*(ldns_buffer_current(rd_buf)) == ' '
+					*(ldns_buffer_current(rd_buf)) == ' ' && !quoted
 				      ) {
 					ldns_buffer_skip(rd_buf, 1);
 				}
 
+				pre_data_pos = ldns_buffer_position(rd_buf);
 				if ((c = ldns_bget_token(rd_buf, rd, delimiters,
 							LDNS_MAX_RDFLEN)) != -1) {
 					/* hmmz, rfc3597 specifies that any type can be represented with
@@ -408,12 +393,26 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 					rd_strlen = strlen(rd);
 
 					/* unknown RR data */
-					if (rd_strlen == 2 && strncmp(rd, "\\#", 2) == 0 && !quoted) {
+					if (strncmp(rd, "\\#", 2) == 0 && !quoted && (rd_strlen == 2 || rd[2]==' ')) {
+                                        	uint16_t hex_data_size;
+                                                char *hex_data_str;
+                                                uint16_t cur_hex_data_size;
+
+                                                was_unknown_rr_format = 1;
+                                                /* go back to before \# and skip it while setting delimiters better */
+                                                ldns_buffer_set_position(rd_buf, pre_data_pos);
+					        delimiters = "\n\t ";
+                                                (void)ldns_bget_token(rd_buf, rd, delimiters, LDNS_MAX_RDFLEN);
+                                                /* read rdata octet length */
 						c = ldns_bget_token(rd_buf, rd, delimiters, LDNS_MAX_RDFLEN);
 						if (c == -1) {
 							/* something goes very wrong here */
-							ldns_buffer_free(rd_buf);
-							LDNS_FREE(rd);
+                                                        LDNS_FREE(rd);
+                                                        LDNS_FREE(b64);
+                                                        ldns_buffer_free(rd_buf);
+                                                        ldns_buffer_free(rr_buf);
+                                                        LDNS_FREE(rdata);
+                                                        ldns_rr_free(new);
 							return LDNS_STATUS_SYNTAX_RDATA_ERR;
 						}
 						hex_data_size = (uint16_t) atoi(rd);
@@ -421,13 +420,27 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 						hex_data_str = LDNS_XMALLOC(char, 2 * hex_data_size + 1);
 						if (!hex_data_str) {
 							/* malloc error */
-							ldns_buffer_free(rd_buf);
-							LDNS_FREE(rd);
+                                                        LDNS_FREE(rd);
+                                                        LDNS_FREE(b64);
+                                                        ldns_buffer_free(rd_buf);
+                                                        ldns_buffer_free(rr_buf);
+                                                        LDNS_FREE(rdata);
+                                                        ldns_rr_free(new);
 							return LDNS_STATUS_SYNTAX_RDATA_ERR;
 						}
 						cur_hex_data_size = 0;
 						while(cur_hex_data_size < 2 * hex_data_size) {
 							c = ldns_bget_token(rd_buf, rd, delimiters, LDNS_MAX_RDFLEN);
+							if (c == -1) {
+								LDNS_FREE(hex_data_str);
+								LDNS_FREE(rd);
+								LDNS_FREE(b64);
+								ldns_buffer_free(rd_buf);
+								ldns_buffer_free(rr_buf);
+								LDNS_FREE(rdata);
+								ldns_rr_free(new);
+								return LDNS_STATUS_SYNTAX_RDATA_ERR;
+							}
 							rd_strlen = strlen(rd);
 							strncpy(hex_data_str + cur_hex_data_size, rd, rd_strlen);
 							cur_hex_data_size += rd_strlen;
@@ -437,17 +450,57 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 						/* correct the rdf type */
 						/* if *we* know the type, interpret it as wireformat */
 						if (desc) {
-							hex_data = LDNS_XMALLOC(uint8_t, hex_data_size + 2);
+							size_t hex_pos = 0;
+							uint8_t *hex_data = LDNS_XMALLOC(uint8_t, hex_data_size + 2);
+                                                        ldns_status s;
+                                                        if(!hex_data) {
+                                                                LDNS_FREE(hex_data_str);
+                                                                LDNS_FREE(rd);
+                                                                LDNS_FREE(b64);
+                                                                ldns_buffer_free(rd_buf);
+                                                                ldns_buffer_free(rr_buf);
+                                                                LDNS_FREE(rdata);
+                                                                ldns_rr_free(new);
+                                                                return LDNS_STATUS_MEM_ERR;
+                                                        }
 							ldns_write_uint16(hex_data, hex_data_size);
 							ldns_hexstring_to_data(hex_data + 2, hex_data_str);
-							hex_pos = 0;
-							(void) ldns_wire2rdf(new, hex_data,
+							s = ldns_wire2rdf(new, hex_data,
 							                 hex_data_size+2, &hex_pos);
+                                                        if(s != LDNS_STATUS_OK) {
+                                                                LDNS_FREE(hex_data_str);
+                                                                LDNS_FREE(rd);
+                                                                LDNS_FREE(b64);
+                                                                ldns_buffer_free(rd_buf);
+                                                                ldns_buffer_free(rr_buf);
+                                                                LDNS_FREE(rdata);
+                                                                ldns_rr_free(new);
+                                                                return s;
+                                                        }
 							LDNS_FREE(hex_data);
 						} else {
 							r = ldns_rdf_new_frm_str(LDNS_RDF_TYPE_HEX, hex_data_str);
+                                                        if(!r) {
+                                                                LDNS_FREE(hex_data_str);
+                                                                LDNS_FREE(rd);
+                                                                LDNS_FREE(b64);
+                                                                ldns_buffer_free(rd_buf);
+                                                                ldns_buffer_free(rr_buf);
+                                                                LDNS_FREE(rdata);
+                                                                ldns_rr_free(new);
+                                                                return LDNS_STATUS_MEM_ERR;
+                                                        }
 							ldns_rdf_set_type(r, LDNS_RDF_TYPE_UNKNOWN);
-							ldns_rr_push_rdf(new, r);
+							if(!ldns_rr_push_rdf(new, r)) {
+                                                                LDNS_FREE(hex_data_str);
+                                                                LDNS_FREE(rd);
+                                                                LDNS_FREE(b64);
+                                                                ldns_buffer_free(rd_buf);
+                                                                ldns_buffer_free(rr_buf);
+                                                                LDNS_FREE(rdata);
+                                                                ldns_rr_free(new);
+                                                                return LDNS_STATUS_MEM_ERR;
+                                                        }
 						}
 						LDNS_FREE(hex_data_str);
 					} else {
@@ -492,6 +545,12 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 								}
 							} else if (r && rd_strlen >= 1 && !ldns_dname_str_absolute(rd) && origin) {
 								if (ldns_dname_cat(r, origin) != LDNS_STATUS_OK) {
+							                LDNS_FREE(rd);
+							                LDNS_FREE(b64);
+							                ldns_buffer_free(rd_buf);
+							                ldns_buffer_free(rr_buf);
+							                LDNS_FREE(rdata);
+							                ldns_rr_free(new);
 									return LDNS_STATUS_ERR;
 								}
 							}
@@ -532,7 +591,7 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 	ldns_buffer_free(rr_buf);
 	LDNS_FREE(rdata);
 
-	if (!question && desc && ldns_rr_rd_count(new) < r_min) {
+	if (!question && desc && !was_unknown_rr_format && ldns_rr_rd_count(new) < r_min) {
 		ldns_rr_free(new);
 		return LDNS_STATUS_SYNTAX_MISSING_VALUE_ERR;
 	}
@@ -541,6 +600,18 @@ ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
 		*newrr = new;
 	}
 	return LDNS_STATUS_OK;
+
+ldnserror:
+	LDNS_FREE(type);
+	LDNS_FREE(owner);
+	LDNS_FREE(ttl);
+	LDNS_FREE(clas);
+	LDNS_FREE(rdata);
+	LDNS_FREE(rd);
+	LDNS_FREE(rd_buf);
+	LDNS_FREE(b64);
+	ldns_rr_free(new);
+    return status;
 }
 
 ldns_status
@@ -580,13 +651,12 @@ ldns_rr_new_frm_fp_l(ldns_rr **newrr, FILE *fp, uint32_t *default_ttl, ldns_rdf
 	char *line;
 	const char *endptr;  /* unused */
 	ldns_rr *rr;
-	char *keyword;
 	uint32_t ttl;
 	ldns_rdf *tmp;
 	ldns_status s;
 	ssize_t size;
+	int offset = 0;
 
-	s = LDNS_STATUS_ERR;
 	if (default_ttl) {
 		ttl = *default_ttl;
 	} else {
@@ -616,12 +686,16 @@ ldns_rr_new_frm_fp_l(ldns_rr **newrr, FILE *fp, uint32_t *default_ttl, ldns_rdf
 		return LDNS_STATUS_SYNTAX_EMPTY;
 	}
 
-	if ((keyword = strstr(line, "$ORIGIN "))) {
+	if (strncmp(line, "$ORIGIN", 7) == 0 && isspace(line[7])) {
 		if (*origin) {
 			ldns_rdf_deep_free(*origin);
 			*origin = NULL;
 		}
-		tmp = ldns_rdf_new_frm_str(LDNS_RDF_TYPE_DNAME, keyword + 8);
+		offset = 8;
+		while (isspace(line[offset])) {
+			offset++;
+		}
+		tmp = ldns_rdf_new_frm_str(LDNS_RDF_TYPE_DNAME, line + offset);
 		if (!tmp) {
 			/* could not parse what next to $ORIGIN */
 			LDNS_FREE(line);
@@ -629,11 +703,17 @@ ldns_rr_new_frm_fp_l(ldns_rr **newrr, FILE *fp, uint32_t *default_ttl, ldns_rdf
 		}
 		*origin = tmp;
 		s = LDNS_STATUS_SYNTAX_ORIGIN;
-	} else if ((keyword = strstr(line, "$TTL "))) {
+	} else if (strncmp(line, "$TTL", 4) == 0 && isspace(line[4])) {
+		offset = 5;
+		while (isspace(line[offset])) {
+			offset++;
+		}
 		if (default_ttl) {
-			*default_ttl = ldns_str2period(keyword + 5, &endptr);
+			*default_ttl = ldns_str2period(line + offset, &endptr);
 		}
 		s = LDNS_STATUS_SYNTAX_TTL;
+	} else if (strncmp(line, "$INCLUDE", 8) == 0) {
+		s = LDNS_STATUS_SYNTAX_INCLUDE;
 	} else {
 		if (origin && *origin) {
 			s = ldns_rr_new_frm_str(&rr, (const char*) line, ttl, *origin, prev);
@@ -689,11 +769,9 @@ ldns_rr_set_rdf(ldns_rr *rr, const ldns_rdf *f, size_t position)
 {
 	size_t rd_count;
 	ldns_rdf *pop;
-	ldns_rdf **rdata_fields;
 
 	rd_count = ldns_rr_rd_count(rr);
 	if (position < rd_count) {
-		rdata_fields = rr->_rdata_fields;
 		/* dicard the old one */
 		pop = rr->_rdata_fields[position];
 		rr->_rdata_fields[position] = (ldns_rdf*)f;
@@ -731,6 +809,7 @@ ldns_rr_pop_rdf(ldns_rr *rr)
 {
 	size_t rd_count;
 	ldns_rdf *pop;
+	ldns_rdf** newrd;
 
 	rd_count = ldns_rr_rd_count(rr);
 
@@ -740,9 +819,15 @@ ldns_rr_pop_rdf(ldns_rr *rr)
 
 	pop = rr->_rdata_fields[rd_count - 1];
 
-	/* shrink the array */
-	rr->_rdata_fields = LDNS_XREALLOC(
-		rr->_rdata_fields, ldns_rdf *, rd_count - 1);
+	/* try to shrink the array */
+	if(rd_count > 1) {
+		newrd = LDNS_XREALLOC(
+			rr->_rdata_fields, ldns_rdf *, rd_count - 1);
+		if(newrd)
+			rr->_rdata_fields = newrd;
+	} else {
+		LDNS_FREE(rr->_rdata_fields);
+	}
 
 	ldns_rr_set_rd_count(rr, rd_count - 1);
 	return pop;
@@ -843,6 +928,7 @@ ldns_rr_list *
 ldns_rr_list_new()
 {
 	ldns_rr_list *rr_list = LDNS_MALLOC(ldns_rr_list);
+        if(!rr_list) return NULL;
 	rr_list->_rr_count = 0;
 	rr_list->_rr_capacity = 0;
 	rr_list->_rrs = NULL;
@@ -878,12 +964,9 @@ bool
 ldns_rr_list_cat(ldns_rr_list *left, ldns_rr_list *right)
 {
 	size_t r_rr_count;
-	size_t l_rr_count;
 	size_t i;
 
-	if (left) {
-		l_rr_count = ldns_rr_list_rr_count(left);
-	} else {
+	if (!left) {
 		return false;
 	}
 
@@ -908,8 +991,6 @@ ldns_rr_list_cat_clone(ldns_rr_list *left, ldns_rr_list *right)
 	size_t i;
 	ldns_rr_list *cat;
 
-	l_rr_count = 0;
-
 	if (left) {
 		l_rr_count = ldns_rr_list_rr_count(left);
 	} else {
@@ -1038,9 +1119,13 @@ ldns_rr_list_pop_rr(ldns_rr_list *rr_list)
 
 	/* shrink the array */
 	if(cap > LDNS_RRLIST_INIT && rr_count-1 <= cap/2) {
+                ldns_rr** a;
 		cap /= 2;
-		rr_list->_rrs = LDNS_XREALLOC(rr_list->_rrs, ldns_rr *, cap);
-		rr_list->_rr_capacity = cap;
+                a = LDNS_XREALLOC(rr_list->_rrs, ldns_rr *, cap);
+                if(a) {
+		        rr_list->_rrs = a;
+		        rr_list->_rr_capacity = cap;
+                }
 	}
 
 	ldns_rr_list_set_rr_count(rr_list, rr_count - 1);
@@ -1257,6 +1342,7 @@ ldns_rr_clone(const ldns_rr *rr)
 	ldns_rr_set_ttl(new_rr, ldns_rr_ttl(rr));
 	ldns_rr_set_type(new_rr, ldns_rr_get_type(rr));
 	ldns_rr_set_class(new_rr, ldns_rr_get_class(rr));
+	ldns_rr_set_question(new_rr, ldns_rr_is_question(rr));
 
 	for (i = 0; i < ldns_rr_rd_count(rr); i++) {
         	if (ldns_rr_rdf(rr,i)) {
@@ -1340,7 +1426,8 @@ qsort_schwartz_rr_compare(const void *a, const void *b)
 			ldns_rr2canonical(canonical_a);
 			sa->transformed_object = ldns_buffer_new(ldns_rr_uncompressed_size(canonical_a));
 			if (ldns_rr2buffer_wire(sa->transformed_object, canonical_a, LDNS_SECTION_ANY) != LDNS_STATUS_OK) {
-				fprintf(stderr, "ERR!\n");
+		                ldns_buffer_free((ldns_buffer *)sa->transformed_object);
+                                sa->transformed_object = NULL;
 				ldns_rr_free(canonical_a);
 				return 0;
 			}
@@ -1351,7 +1438,10 @@ qsort_schwartz_rr_compare(const void *a, const void *b)
 			ldns_rr2canonical(canonical_b);
 			sb->transformed_object = ldns_buffer_new(ldns_rr_uncompressed_size(canonical_b));
 			if (ldns_rr2buffer_wire(sb->transformed_object, canonical_b, LDNS_SECTION_ANY) != LDNS_STATUS_OK) {
-				fprintf(stderr, "ERR!\n");
+		                ldns_buffer_free((ldns_buffer *)sa->transformed_object);
+		                ldns_buffer_free((ldns_buffer *)sb->transformed_object);
+                                sa->transformed_object = NULL;
+                                sb->transformed_object = NULL;
 				ldns_rr_free(canonical_b);
 				return 0;
 			}
@@ -1378,8 +1468,18 @@ ldns_rr_list_sort(ldns_rr_list *unsorted)
 
 		sortables = LDNS_XMALLOC(struct ldns_schwartzian_compare_struct *,
 					 item_count);
+                if(!sortables) return; /* no way to return error */
 		for (i = 0; i < item_count; i++) {
 			sortables[i] = LDNS_XMALLOC(struct ldns_schwartzian_compare_struct, 1);
+                        if(!sortables[i]) {
+                                /* free the allocated parts */
+                                while(i>0) {
+                                        i--;
+                                        LDNS_FREE(sortables[i]);
+                                }
+                                /* no way to return error */
+                                return;
+                        }
 			sortables[i]->original_object = ldns_rr_list_rr(unsorted, i);
 			sortables[i]->transformed_object = NULL;
 		}
@@ -1801,13 +1901,16 @@ static const ldns_rdf_type type_nsec_wireformat[] = {
 static const ldns_rdf_type type_dhcid_wireformat[] = {
 	LDNS_RDF_TYPE_B64
 };
+static const ldns_rdf_type type_talink_wireformat[] = {
+	LDNS_RDF_TYPE_DNAME, LDNS_RDF_TYPE_DNAME
+};
 /* nsec3 is some vars, followed by same type of data of nsec */
 static const ldns_rdf_type type_nsec3_wireformat[] = {
 /*	LDNS_RDF_TYPE_NSEC3_VARS, LDNS_RDF_TYPE_NSEC3_NEXT_OWNER, LDNS_RDF_TYPE_NSEC*/
 	LDNS_RDF_TYPE_INT8, LDNS_RDF_TYPE_INT8, LDNS_RDF_TYPE_INT16, LDNS_RDF_TYPE_NSEC3_SALT, LDNS_RDF_TYPE_NSEC3_NEXT_OWNER, LDNS_RDF_TYPE_NSEC
 };
 
-static const ldns_rdf_type type_nsec3params_wireformat[] = {
+static const ldns_rdf_type type_nsec3param_wireformat[] = {
 /*	LDNS_RDF_TYPE_NSEC3_PARAMS_VARS*/
 	LDNS_RDF_TYPE_INT8,
 	LDNS_RDF_TYPE_INT8,
@@ -1941,7 +2044,7 @@ static ldns_rr_descriptor rdata_field_descriptors[] = {
 	/* 50 */
 	{LDNS_RR_TYPE_NSEC3, "NSEC3", 5, 6, type_nsec3_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 	/* 51 */
-{LDNS_RR_TYPE_NSEC3PARAMS, "NSEC3PARAM", 4, 4, type_nsec3params_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
+{LDNS_RR_TYPE_NSEC3PARAM, "NSEC3PARAM", 4, 4, type_nsec3param_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 	/* 52 */
 {LDNS_RR_TYPE_NULL, "TYPE52", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE53", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
@@ -1949,7 +2052,7 @@ static ldns_rr_descriptor rdata_field_descriptors[] = {
 {LDNS_RR_TYPE_NULL, "TYPE55", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE56", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE57", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
-{LDNS_RR_TYPE_NULL, "TYPE58", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
+{LDNS_RR_TYPE_TALINK, "TALINK", 2, 2, type_talink_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 2 },
 {LDNS_RR_TYPE_NULL, "TYPE59", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE60", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE61", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
