@@ -32,7 +32,7 @@ xprintf_rdf(ldns_rdf *rd)
 	/* assume printable string */
 	fprintf(stderr, "size\t:%u\n", (unsigned int)ldns_rdf_size(rd));
 	fprintf(stderr, "type\t:%u\n", (unsigned int)ldns_rdf_get_type(rd));
-	fprintf(stderr, "data\t:[%.*s]\n", (int)ldns_rdf_size(rd), 
+	fprintf(stderr, "data\t:[%.*s]\n", (int)ldns_rdf_size(rd),
 			(char*)ldns_rdf_data(rd));
 }
 
@@ -84,7 +84,7 @@ ldns_lookup_by_id(ldns_lookup_table *table, int id)
 	return NULL;
 }
 
-int 
+int
 ldns_get_bit(uint8_t bits[], size_t index)
 {
 	/*
@@ -94,7 +94,7 @@ ldns_get_bit(uint8_t bits[], size_t index)
 	return (int) (bits[index / 8] & (1 << (7 - index % 8)));
 }
 
-int 
+int
 ldns_get_bit_r(uint8_t bits[], size_t index)
 {
 	/*
@@ -105,13 +105,13 @@ ldns_get_bit_r(uint8_t bits[], size_t index)
 }
 
 void
-ldns_set_bit(uint8_t *byte, int bit_nr, bool value) 
+ldns_set_bit(uint8_t *byte, int bit_nr, bool value)
 {
 	if (bit_nr >= 0 && bit_nr < 8) {
 		if (value) {
 			*byte = *byte | (0x01 << bit_nr);
 		} else {
-			*byte = *byte & !(0x01 << bit_nr);
+			*byte = *byte & ~(0x01 << bit_nr);
 		}
 	}
 }
@@ -141,7 +141,7 @@ ldns_hexdigit_to_int(char ch)
 	}
 }
 
-char 
+char
 ldns_int_to_hexdigit(int i)
 {
 	switch (i) {
@@ -180,7 +180,7 @@ ldns_hexstring_to_data(uint8_t *data, const char *str)
 	}
 
 	for (i = 0; i < strlen(str) / 2; i++) {
-		data[i] = 
+		data[i] =
 			16 * (uint8_t) ldns_hexdigit_to_int(str[i*2]) +
 			(uint8_t) ldns_hexdigit_to_int(str[i*2 + 1]);
 	}
@@ -199,7 +199,7 @@ static const int mdays[] = {
 	31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
 };
 
-static int 
+static int
 is_leap_year(int year)
 {
 	return year % 4 == 0 && (year % 100 != 0 || year % 400 == 0);
@@ -241,6 +241,100 @@ mktime_from_utc(const struct tm *tm)
 	return seconds;
 }
 
+#if SIZEOF_TIME_T <= 4
+
+static void
+ldns_year_and_yday_from_days_since_epoch(int64_t days, struct tm *result)
+{
+	int year = 1970;
+	int new_year;
+
+	while (days < 0 || days >= (int64_t) (is_leap_year(year) ? 366 : 365)) {
+		new_year = year + (int) (days / 366);
+		if (year == new_year) {
+			year += days < 0 ? -1 : 1;
+		}
+		days -= (new_year - year) * 365;
+		days -= leap_days(year, new_year);
+		year  = new_year;
+	}
+	result->tm_year = year;
+	result->tm_yday = (int) days;
+}
+
+/* Number of days per month in a leap year. */
+static const int leap_year_mdays[] = {
+	31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
+};
+
+static void
+ldns_mon_and_mday_from_year_and_yday(struct tm *result)
+{
+	int idays = result->tm_yday;
+	const int *mon_lengths = is_leap_year(result->tm_year) ? 
+					leap_year_mdays : mdays;
+
+	result->tm_mon = 0;
+	while  (idays >= mon_lengths[result->tm_mon]) {
+		idays -= mon_lengths[result->tm_mon++];
+	}
+	result->tm_mday = idays + 1;
+}
+
+static void
+ldns_wday_from_year_and_yday(struct tm *result)
+{
+	result->tm_wday  = 4 /* 1-1-1970 was a thursday */
+			 + ((result->tm_year - 1970) % 7) * (365 % 7)
+			 + leap_days(1970, result->tm_year)
+			 + result->tm_yday;
+	result->tm_wday %= 7;
+	if (result->tm_wday < 0) {
+		result->tm_wday += 7;
+	}
+}
+
+static struct tm *
+ldns_gmtime64_r(int64_t clock, struct tm *result)
+{
+	result->tm_isdst =                 0;
+	result->tm_sec   = (int) (clock % 60);
+	clock           /=                60;
+	result->tm_min   = (int) (clock % 60);
+	clock           /=                60;
+	result->tm_hour  = (int) (clock % 24);
+	clock           /=                24;
+
+	ldns_year_and_yday_from_days_since_epoch(clock, result);
+	ldns_mon_and_mday_from_year_and_yday(result);
+	ldns_wday_from_year_and_yday(result);
+	result->tm_year -= 1900;
+
+	return result;
+}
+
+#endif /* SIZEOF_TIME_T <= 4 */
+
+static int64_t
+ldns_serial_arithmitics_time(int32_t time, time_t now)
+{
+	int32_t offset = time - (int32_t) now;
+	return (int64_t) now + offset;
+}
+
+
+struct tm *
+ldns_serial_arithmitics_gmtime_r(int32_t time, time_t now, struct tm *result)
+{
+#if SIZEOF_TIME_T <= 4
+	int64_t secs_since_epoch = ldns_serial_arithmitics_time(time, now);
+	return  ldns_gmtime64_r(secs_since_epoch, result);
+#else
+	time_t  secs_since_epoch = ldns_serial_arithmitics_time(time, now);
+	return  gmtime_r(&secs_since_epoch, result);
+#endif
+}
+
 /**
  * Init the random source
  * applications should call this if they need entropy data within ldns
@@ -253,7 +347,7 @@ mktime_from_utc(const struct tm *tm)
  * If openssl *is* available, this function just adds more entropy
  **/
 int
-ldns_init_random(FILE *fd, unsigned int size) 
+ldns_init_random(FILE *fd, unsigned int size)
 {
 	/* if fp is given, seed srandom with data from file
 	   otherwise use /dev/urandom */
@@ -262,15 +356,17 @@ ldns_init_random(FILE *fd, unsigned int size)
 	size_t read = 0;
 	unsigned int seed_i;
 	struct timeval tv;
-	struct timezone tz;
 
 	/* we'll need at least sizeof(unsigned int) bytes for the
 	   standard prng seed */
 	if (size < (unsigned int) sizeof(seed_i)){
 		size = (unsigned int) sizeof(seed_i);
 	}
-	
+
 	seed = LDNS_XMALLOC(uint8_t, size);
+        if(!seed) {
+		return 1;
+        }
 
 	if (!fd) {
 		if ((rand_f = fopen("/dev/urandom", "r")) == NULL) {
@@ -279,7 +375,7 @@ ldns_init_random(FILE *fd, unsigned int size)
 				/* no readable /dev/random either, and no entropy
 				   source given. we'll have to improvise */
 				for (read = 0; read < size; read++) {
-					gettimeofday(&tv, &tz);
+					gettimeofday(&tv, NULL);
 					seed[read] = (uint8_t) (tv.tv_usec % 256);
 				}
 			} else {
@@ -292,7 +388,7 @@ ldns_init_random(FILE *fd, unsigned int size)
 		rand_f = fd;
 		read = fread(seed, 1, size, rand_f);
 	}
-	
+
 	if (read < size) {
 		LDNS_FREE(seed);
 		return 1;
@@ -310,14 +406,32 @@ ldns_init_random(FILE *fd, unsigned int size)
 #endif
 		LDNS_FREE(seed);
 	}
-	
+
 	if (!fd) {
-		fclose(rand_f);
+                if (rand_f) fclose(rand_f);
 	}
 
 	return 0;
 }
 
+/**
+ * Get random number.
+ *
+ */
+uint16_t
+ldns_get_random(void)
+{
+        uint16_t rid = 0;
+#ifdef HAVE_SSL
+        if (RAND_bytes((unsigned char*)&rid, 2) != 1) {
+                rid = (uint16_t) random();
+        }
+#else
+        rid = (uint16_t) random();
+#endif
+	return rid;
+}
+
 /*
  * BubbleBabble code taken from OpenSSH
  * Copyright (c) 2001 Carsten Raskgaard.  All rights reserved.
@@ -333,6 +447,7 @@ ldns_bubblebabble(uint8_t *data, size_t len)
 
 	rounds = (len / 2) + 1;
 	retval = LDNS_XMALLOC(char, rounds * 6);
+	if(!retval) return NULL;
 	retval[j++] = 'x';
 	for (i = 0; i < rounds; i++) {
 		size_t idx0, idx1, idx2, idx3, idx4;
